#include "CustomContainer.hpp"
#include "ItsWindow.hpp"
#include "ColorDef.hpp"
#include "helloworld.brh"

// constructor
CustomContainer::CustomContainer(Void) static_throws
{
    if (static_try()) {
        SetType(CODE_TYPE);
    }
}

// destructor
CustomContainer::~CustomContainer(Void)
{
}

// create CustomContainer instance
CustomContainerSmp CustomContainer::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<CustomContainer>(Factory(:: new CustomContainer, exception));
}

// drawing handler of CustomContainer
Void CustomContainer::HandleRenderRequest(SFXGraphicsPtr graphics) const
{
    // line type
    AEEStrokeStyle style;

    // save line type before drawing
    style = graphics->GetStrokeStyle();

    // set line type to dotted line
    graphics->SetStrokeStyle(STROKE_DOTTED);

    // calculate visible region in CustomContainer
    // visible region is real region of CustomContainer
    SFXRectangle rx;
    rx.Set(GetVirtualBound());
    rx.Set(SFXGrid(-rx.GetX(), -rx.GetY()), GetRealBound().GetSize());

    // draw black dotted line on border of visible region
    graphics->DrawRectangle(rx, COLOR_BLACK);

    // restore line type
    graphics->SetStrokeStyle(style);

    return;
}

// make flex list box control
SFCError CustomContainer::Make(Void)
{
    SFZFlexListBoxControlSmp list;
    SFZFlexListBoxControl::ItemRec item;
    SFBShellSmp shell(SFBShell::GetInstance());
    SFBImageSmp normal(shell->LoadResImage("helloworld.bar", IDI_OBJECT_5001));
    SFBImageSmp select(shell->LoadResImage("helloworld.bar", IDI_OBJECT_5002));
    SFCError error(SFERR_NO_ERROR);

    // create flex list box control
    if ((list = SFZFlexListBoxControl::NewInstance(&error)) != null) {

        // set flex list box control's parent responder to container
        if ((error = list->SetParent(GetThis())) == SFERR_NO_ERROR) {

            // register result handler into flex list box control
            error = list->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnListResult)
            );

            if (error == SFERR_NO_ERROR) {

                // set background color of even numbered items
                list->SetEvenListColor(SFXRGBColor(0xFF, 0xFF, 0xCC, 0x00));

                // set background color of odd numbered items
                list->SetOddListColor(SFXRGBColor(0xFF, 0xCC, 0xFF, 0x00));

                // set horizontal alignment for nomal item to "center-aligned" 
                list->SetHorizontalAlign(SFZFlexListBoxControl::NORMAL, SFZFlexListBoxControl::HORIZONTAL_CENTER);

                // set repeat-scroll flag to "false"
                list->SetScrollRepeat(false);

                // set scroll bar's width
                list->SetScrollBarWidth(5);

                // set that multiple string is displayed when selected
                list->SetToggle(SFZFlexListBoxControl::TOGGLE_MULTIPLE_BODY_TEXT);

                SFYResponderSmp parent;
                if ((parent = GetParent()) != null) {

                    // if type of container's parent is not that of tab control
                    if (parent->GetType() != SFZTabControl::CODE_TYPE) {

                        // set flex list box control's move-item-up key to LEFT key
                        list->SetScrollUpKey(AVK_LEFT);

                        // set flex list box control's move-item-down key to RIGHT key
                        list->SetScrollDownKey(AVK_RIGHT);
                    }
                }

                // set access key enable
                list->SetAccessKeyEnable(true);

                // append item using item structure
                // though only string can be appended,
                // using item structure is recommended to append image

                // set enable flag of item structure to "true"
                item.property.enable = true;

                // set direct access key and command ID of item structure
                // (ID is not used in this case)
                item.property.key = 0;
                item.property.id = SFZFlexListBoxControl::INVALID_COMMAND_ID;

                // set image of item structure
                item.normal.body.icon = normal;
                item.select.body.icon = select;

                // append 8 items
                for (SInt32 i = 0;  i < 8; ++i) {

                    // set text of item structure
                    item.normal.header.text.Set(SFXWideString::Format("%02d", i));
                    item.normal.body.text.Set(SFXWideString::Format("List item %02d", i));
                    item.select.body.text.Set(SFXWideString::Format("The item %02d is selected.", i));

                    // set access key of item structure
                    item.property.key = AVK_0 + i;

                    // insert at tail
                    if ((error = list->InsertLast(item)) != SFERR_NO_ERROR) {
                        break;
                    }
                }
                if (error == SFERR_NO_ERROR) {

                    // set flex list box control's real region
                    list->SetRealBound(list->GetSuitableBound(GetLocalBound().Deflate(10, 10), SFYResponder::HORIZONTAL_CENTER, SFYResponder::VERTICAL_TOP));

                    // set flex list box control's state to "visible" + "active" + "enable" + "focus" together
                    list->SetState(true, true, true, true);

                    list->SetCurrentValue(1);

                    // (*)
                    // set current value after setting enable state to "true" 
                    //
                }
            }
        }
    }
    return error;
}

// result handler of flex list box control
XANDLER_IMPLEMENT_VOIDRESULT(CustomContainer, OnListResult, invoker, reason, result)
{
    SFYResponderSmp parent;

    unused(invoker);
    unused(result);
    switch (reason) {
        case SFP16_RESULT_ESCAPE:
            if ((parent = GetParent()) != null) {

                // if type of container's parent is that of tab control
                if (parent->GetType() == SFZTabControl::CODE_TYPE) {

                    // terminate container 
                    Terminate();

                }

                // if container's parent is ItsWindow
                else {

                    // send CLEAR key event to container's parent
                    parent->InvokeForward(SFXEvent(SFEVT_KEY, AVK_CLR, 0), false);

                }
            }
            break;
    }
    return;
}
