#include "ItsWindow.hpp"
#include "CustomContainer.hpp"
#include "ColorDef.hpp"

#define COLOR_ITS_WINDOW_BACK (SFXRGBColor(0xCC, 0xFF, 0xCC, 0x00))

// constructor
ItsWindow::ItsWindow(Void) static_throws
{
    if (static_try()) {
        SetType(CODE_TYPE);

        // register key handler into ItsWindow
        static_throw(RegisterHandler(
            SFXEventRange(SFEVT_KEY, SFEVT_KEY, SFP16_BEGIN, SFP16_END),
            XANDLER_INTERNAL(OnKey)
        ));

        // set ItsWindows's background color to light green
        SetBackgroundColor(COLOR_ITS_WINDOW_BACK);
    }
}

// destructor
ItsWindow::~ItsWindow(Void)
{
}

// create ItsWindow instance
ItsWindowSmp ItsWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<ItsWindow>(Factory(:: new ItsWindow, exception));
}

// make container and controls on ItsWindow
SFCError ItsWindow::Make(Void)
{
    CustomContainerSmp container;
    SFZContainerScrollBarControlSmp bar;
    SFCError error(SFERR_NO_ERROR);

    #if 0
    // make general purpose container
    if ((container = SFZContainer::NewInstance(&error)) != null) {
    #endif

    // make custom container
    if ((container = CustomContainer::NewInstance(&error)) != null) {

        // set container's parent responder to window
        if ((error = container->SetParent(GetThis())) == SFERR_NO_ERROR) {

            // set container's real region to region obtained by deflating window's local region by (5, 5, 10, 5)
            container->SetRealBound(GetLocalBound().Deflate(5, 5, 10, 5));

            // set container's virtual region to region obtained by expanding container's real region down by 150
            container->SetVirtualBound(container->GetLocalBound().AddHeight(150));

            // set container's state to "visible" + "active" + "enable" + "focus" together 
            container->SetState(true, true, true, true);

            // make and place controls in container
            container->Make();

            //#if 0

            // in case button is placed 
            SFZTextButtonControlSmp button;

            // create button control
            if ((button = SFZTextButtonControl::NewInstance(&error)) != null) {

                // set button control's parent responder to container
                if ((error = button->SetParent(container)) == SFERR_NO_ERROR) {

                    // set button control's label to "CenterMiddle"
                    if ((error = button->SetText("CenterMiddle")) == SFERR_NO_ERROR) {

                        // set button control's real region in center of container's local region
                        button->SetRealBound(button->GetSuitableBound(container->GetLocalBound(), SFYResponder::HORIZONTAL_CENTER, SFYResponder::VERTICAL_MIDDLE));

                        // set button control's state to "visible" + "active" + "enable" + "focus" together
                        button->SetState(true, true, true, true);
                    }
                }
            }

            //#endif
        }
    }
    if (error == SFERR_NO_ERROR) {

        // create scroll bar control for container
        if ((bar = SFZContainerScrollBarControl::NewInstance(&error)) != null) {

            // set parent responder to window
            if ((error = bar->SetParent(GetThis())) == SFERR_NO_ERROR) {

                // place scroll bar control on right of container
                // * scroll bar's width is 5
                SFXRectangle rx;
                rx.Set(container->GetRealBound());
                rx.AddX(rx.GetWidth());
                rx.SetWidth(5);
                bar->SetRealBound(rx);

                // set scroll bar control's state to "visible" + "active" + "unenable" + "unfocus" together
                // * scroll bar control must be never focused
                bar->SetState(true, true, false, false);

                /// bind container with scroll bar control
                error = bar->Bind(container);
            }
        }
    }
    return error;
}

// key handler
XANDLER_IMPLEMENT_BOOLEVENT(ItsWindow, OnKey, invoker, event)
{
    Bool result(false);

    unused(invoker);
    switch (event.GetP16()) {

        // when CLEAR key is pressed
        case AVK_CLR:

            // terminate window
            Terminate();

            result = true;
            break;
    }
    return result;
}
