#ifndef __MYWINDOW_HPP
#define __MYWINDOW_HPP

#define TARGET_BUILD_DEBUG

#include <SophiaFramework.hpp>

//
// MyWindow class (place various controls)
//

// macro for generating useful types
SFMTYPEDEFRESPONDER(MyWindow)

#if 0
// inherit from SFZWindow
class MyWindow : public SFZWindow
#endif
// inherit from SFZTabPage
class MyWindow : public SFZTabPage
{
    // this macro prohibits copying instance
    SFMSEALRESPONDER(MyWindow)

    // this macro specifies inheritance relation from SFYResponder to MyWindow
    #if 0
    SFMRESPONDERINSTANTIATEFOUR(MyWindow, SFZWindow, SFYContainer, SFYWidget, SFYResponder)
    #endif
    SFMRESPONDERINSTANTIATEFOUR(MyWindow, SFZTabPage, SFYContainer, SFYWidget, SFYResponder)

public:

    // since small alphabet is reserved for 4-characters literals, 
    // define ('M', 'W', 'N', 'D') as 4-character literals for MyWindow with capital alphabet
    enum CodeEnum {
        CODE_TYPE = four_char_code('M', 'W', 'N', 'D')
    };
    SFMTYPEDEFTYPE(CodeEnum)

private:

    // button control 1
    SFZTextButtonControlSmp _button1;

    // button control 2
    SFZTextButtonControlSmp _button2;

    // button control 3
    SFZTextButtonControlSmp _button3;

    // check box control 1
    SFZCheckboxControlSmp _checkbox1;

    // check box control 2
    SFZCheckboxControlSmp _checkbox2;

    // radio button control 1
    SFZRadiobuttonControlSmp _radio1;

    // radio button control 2
    SFZRadiobuttonControlSmp _radio2;

    // list box control
    SFZComboBoxControlSmp _combobox;

    // label control
    SFZSingleTextLabelControlSmp _singleTextLabel;

    // edit box control
    SFZSingleEditBoxControlSmp _singleEditBox;

    // list box control
    SFZListBoxControlSmp _listbox;

    // switching flag
    Bool _ifShowHw;

public:

    // generate MyWindow instance managed by smart pointer
    static MyWindowSmp NewInstance(SFCErrorPtr exception = null);

    // make and place various controls on MyWindow
    SFCError Make(Void);

protected:

    // constructor
    explicit MyWindow(Void) static_throws;

    // destructor
    virtual ~MyWindow(Void);

    // drawing handler(override HandleRenderRequest virtual function booted up first when drawing event occurs)
    virtual Void HandleRenderRequest(SFXGraphicsPtr graphics) const;

private:

    // make and place button control
    SFCError CreateButtons(SFXGridPtr offs);

    // make and place check box control
    SFCError CreateCheckBoxes(SFXGridPtr offs);

    // make and place radio button control
    SFCError CreateRadioButtons(SFXGridPtr offs);

    // make and place list box control
    SFCError CreateComboBox(SFXGridPtr offs);

    // make and place label control and edit box control
    SFCError CreatePassLine(SFXGridPtr offs);

    // make and place list box control
    SFCError CreateListBox(SFXGridPtr offs);

    // key handler
    XANDLER_DECLARE_BOOLEVENT(OnKey)

    // result handler for button control 1-2
    XANDLER_DECLARE_VOIDRESULT(OnButton12)

    // result handler for button control 3
    XANDLER_DECLARE_VOIDRESULT(OnButton3)

    // result handler for check box control
    XANDLER_DECLARE_VOIDRESULT(OnCheck)

    // result handler for radio button control
    XANDLER_DECLARE_VOIDRESULT(OnRadio)

    // result handler for list box control
    XANDLER_DECLARE_VOIDRESULT(OnComboBox)

    // value handler for list box control internally contained by combo box control
    XANDLER_DECLARE_VOIDVALUE(OnComboListValue)

    // result handler for list box control
    XANDLER_DECLARE_VOIDRESULT(OnListBox)

    // value handler for list box control
    XANDLER_DECLARE_VOIDVALUE(OnListValue)

    // result handler for edit box control
    XANDLER_DECLARE_VOIDRESULT(OnSingleEditBox)

};

#endif // __MYWINDOW_HPP
