#include "TabWindow.hpp"
#include "MyWindow.hpp"
#include "CustomContainer.hpp"
#include "ColorDef.hpp"
#include "UserEvent.hpp"

#define COLOR_TAB COLOR_WHITE

// constructor
TabWindow::TabWindow(Void)
{
    if (static_try()) {
        SetType(CODE_TYPE);

        // register key handler
        static_throw(RegisterHandler(
            SFXEventRange(SFEVT_KEY, SFEVT_KEY, SFP16_BEGIN, SFP16_END),
            XANDLER_INTERNAL(OnKey)
        ));
    }
}

// destructor
TabWindow::~TabWindow(Void)
{
}

// create instance
TabWindowSmp TabWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<TabWindow>(Factory(:: new TabWindow, exception));
}

// make responder
SFCError TabWindow::Make()
{
    MyWindowSmp myWindow;
    CustomContainerSmp customContainer;
    SFCError error(SFERR_NO_ERROR);

    // create tab control
    if ((_tab = SFZTabControl::NewInstance(&error)) != null) {

        // set tab control's parent responder to TabWindow
        error = _tab->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register color scheme handler into tab control
            error = _tab->RegisterHandler(
                SFXEventRange(SFEVT_USER_COLOR, SFEVT_USER_COLOR, SFP16_USER_COLOR, SFP16_USER_COLOR),
                XANDLER_INTERNAL(OnTabColor)
            );

            if (error == SFERR_NO_ERROR) {

                // set tab control's real region
                _tab->SetRealBound(GetLocalBound());

                // set colors of various tab control's parts
                _tab->SetBackgroundColor(COLOR_TAB);

                // set tab control's border line to be drawn
                _tab->SetDrawBorder(true);

                // undisplay tab control's scroll bar
                _tab->SetScrollBarWidth(0);
                _tab->SetScrollBarControl(SFYScrollBarControlSmp::EmptyInstance());

                // set 2 page to maximum tabs displayed in tab control
                _tab->SetFieldValue(2);

                // set tab control's state to "visible" + "active" + "enable" + "focus" together
                _tab->SetState(true, true, true, true);

                // make MyWindow(MyWindow's inheriting from parent responder is changed into SFZTabPage)
                if ((myWindow = MyWindow::NewInstance(&error)) != null) {

                    // set MyWindow's parent responder to tab control
                    // MyWindow's real region is automatically set to region obtained by excluding tab label and hint region from tab control region 
                    error = myWindow->SetParent(_tab);

                    if (error == SFERR_NO_ERROR) {

                        // set title text of MyWindow as tab page (this text will be dispalyed in tab region)
                        myWindow->SetTitle("Controls");

                        // set hint text of MyWindow as tab page
                        myWindow->SetHint("This is MyWindow");

                        // set MyWindow's state to "visible" + "active" + "enable" + "focus" together 
                        myWindow->SetState(true, true, true, true);

                        // make responders in MyWindow
                        myWindow->Make();

                        // make CustomContainer(CustomContainer's inheriting from parent responder has been changed into SFZTabPage)
                        if ((customContainer = CustomContainer::NewInstance(&error)) != null) {

                            error = customContainer->SetParent(_tab);

                            if (error == SFERR_NO_ERROR) {

                                // set set title text of CustomContainer as tab page (this text will be displayed in tab region)
                                customContainer->SetTitle("Container");

                                // set hint text of CustomContainer as tab page
                                customContainer->SetHint("This is CustomContainer");

                                // set CustomContainer's state to "visible" + "active" + "enable" + "unfocus" together 
                                customContainer->SetState(true, true, true, false);

                                // make responders in CustomContainer
                                customContainer->Make();

                                // calculate suitable virtual region on all tab pages automatically
                                // In this case, MyWindow virtual region will be just extended to
                                // include MyWindow controls because MyWindow real region cannot
                                // include the controls.
                                _tab->AdjustPages();

                                // select 2nd tab page(CustomContainer)
                                _tab->FocusPage(1);
                            }
                        }
                    }
                }
            }
        }
    }
    return error;
}

// key handler
XANDLER_IMPLEMENT_BOOLEVENT(TabWindow, OnKey, invoker, event)
{
    SFYResponderSmp parent;
    Bool result(false);

    unused(invoker);
    switch (event.GetP16()) {
        case AVK_CLR:

            // terminate TabWindow
            Terminate();

            result = true;
            break;
    }
    return result;
}

// color scheme handler
XANDLER_IMPLEMENT_VOIDUSERCOLOR(TabWindow, OnTabColor, invoker, reason, color)
{
    unused(invoker);
    unused(reason);

    // set background color of tab control
    _tab->SetBackgroundColor(color->base);

    // set bevel color of hint region of tab control
    _tab->SetHintBevelColor(SFXBevelColor(color->light, color->base, color->dark));

    // set bevel color of tab control
    _tab->SetTabBevelColor(SFXBevelColor(color->light, color->base, color->dark));

    // get each tab page and set color of title text
    for (SInt16 i = 0; i < _tab->GetPageCount(); ++i) {
        SFZTabPageSmp page = _tab->GetPage(i);
        page->SetTextColor(color->selFore);
    }

    return;
}
