//      helloworld.hpp

#ifndef __HELLOWORLD_HPP
#define __HELLOWORLD_HPP

#define TARGET_BUILD_DEBUG

#include <SophiaFramework.hpp>
#include "helloworld.bid"
#include "UserColor.hpp"
#include "MyWindow.hpp"
#include "ItsWindow.hpp"
#include "TabWindow.hpp"

enum {

    // allocate identifier to each label text for SoftKey menu
    KEY_TEXT_EXIT   = 1,
    KEY_TEXT_SELECT,
    KEY_TEXT_CONFIG,
    KEY_TEXT_BACK

};

enum {

    // allocate identifier to each SoftKey menu
    KEY_MENU_TOP    = 1,
    KEY_MENU_WINDOW,
    KEY_MENU_TEXTMENU,
    KEY_MENU_DIALOG

};

//
//  helloworld application class
//

//  macro for generating useful types
SFMTYPEDEFCLASS(helloworld)

//  helloworld inherits from SFYApplication
class helloworld : public SFYApplication {

    // this macro prohibits copying instance
    SFMSEALCOPY(helloworld)

private:

    // smart pointer to MyWindow
    MyWindowSmp _myWindow;

    // smart pointer to ItsWindow
    ItsWindowSmp _itsWindow;

    // smart pointer to SFZTextMenu
    SFZTextMenuSmp _textMenu;

    // smart pointer to TabWindow
    TabWindowSmp _tabWindow;

    // smart pointer to SFZSoftKeyControl
    SFZSoftKeyControlSmp _softkey;

    // smart pointer to SFZTitlePlainFrame (frame for window)
    SFZTitlePlainFrameSmp _windowFrame;

    // smart pointer to SFZTitleBevelFrame (frame for dialog)
    SFZTitleBevelFrameSmp _dialogFrame;

public:

    // Factory function
    static SFCInvokerPtr Factory(Void);

private:

    // constructor
    explicit helloworld(Void) static_throws;

    // destructor
    virtual ~helloworld(Void);

    // make MyWindow
    SFCError MakeMy(Void);

    // make SFZTextMenu
    SFCError MakeMenu(Void);

    // make ItsWindow
    SFCError MakeIts(Void);

    // make TabWindow
    SFCError MakeTab(Void);

    // make message dialog 
    SFCError MakeColorDialog(SFXWideStringConstRef title, UserColorConstRef color);

    // make SoftKey menu
    SFCError MakeSoftMenu(UInt32 key);

    // make frame for window
    SFCError MakeWindowFrame(SFXWideStringConstRef title);

    // make frame for dialog
    SFCError MakeDialogFrame(SFXWideStringConstRef title);

    // register label texts of SoftKey menu
    SFCError RegisterSoftText(Void);

    // set menu colors together
    Void SetMenuColors(UserColorConstRef color);

    // get region obtained by excluding SoftKey region from device screen
    SFXRectangle GetContentBound(Void) const;

    // drawing handler
    XANDLER_DECLARE_VOIDRENDER(OnRenderRequest)

    // key handler
    XANDLER_DECLARE_BOOLEVENT(OnKey)

    // result handler of menu
    XANDLER_DECLARE_VOIDRESULT(OnMenuResult)

    // SoftKey handler of helloworld
    XANDLER_DECLARE_VOIDEVENT(OnSoftKey)

    // SoftKey handler of text menu
    XANDLER_DECLARE_VOIDEVENT(OnMenuSoftKey)

    // SoftKey handler of window
    XANDLER_DECLARE_VOIDEVENT(OnWindowSoftKey)

    // key handler of window
    XANDLER_DECLARE_BOOLEVENT(OnWindowKey)

    // result handler of dialog
    XANDLER_DECLARE_VOIDRESULT(OnDialogResult)

    // color scheme handler of SoftKey control
    XANDLER_DECLARE_VOIDUSERCOLOR(OnSoftColor)

    // color scheme handler of window frame
    XANDLER_DECLARE_VOIDUSERCOLOR(OnWindowFrameColor)
};

#endif // __HELLOWORLD_HPP //
